package algoritmy;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import utility.DipUtils;
import utility.DipValidators;

/**
 * Trida realizujici LZPR (Linearni zpetnovazebni posuvny registr).
 * Poskytuje (hlavni) metodu na vytvoreni LZPR sekvence a dilci metody
 * pouzite uvnitr metody hlavni.
 */
public class AlgLinZpetVazReg {    

    public static void main(String[] args) throws Exception {
        Properties properties = DipUtils.nactiProperties();
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.detekovane.poruchy"));
        AlgLinZpetVazReg alzvr = new AlgLinZpetVazReg();
        if (args.length != 1) {
            throw new Exception(properties.getProperty("param.lzrp.error"));
        }
        
        if ("deter".equals(args[0])) {
            alzvr.simulujDeterFazi();      
        } else if ("lzpr".equals(args[0])) {
            alzvr.simulujKrokyLZPR(13000);
        } else {
            throw new Exception(properties.getProperty("param.lzrp.error"));
        }       
    }

   /**
    * Metoda simuluje zadany pocet kroku LZPR.
    *
    * @param pocetKroku
    * @throws Exception
    */
    private void simulujKrokyLZPR(int pocetKroku) throws Exception {
        Properties properties = DipUtils.getProperties();
        String polynom;
        String testVektor;
        int stupenPoly = Integer.valueOf(properties.getProperty("stupen.polynomu"));

        if (properties.getProperty("generuj.polynom").equalsIgnoreCase("0")) {
            polynom = properties.getProperty("polynom");
        } else {
            polynom = DipUtils.generujPolynom(stupenPoly);
        }

        if (properties.getProperty("generuj.seed").equalsIgnoreCase("0")) {
            testVektor = properties.getProperty("seed");
        } else {
            testVektor = DipUtils.generujSeed(stupenPoly);
        }
       
        String chybMaska = "";
        String chybMaskaZal = "";
        int aktDetPoruch = 0;
        List<Integer> detekPoruchy = new ArrayList<Integer>();

        for (int i = 0; i < pocetKroku; i++) {
            //DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), testVektor, true);
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.mask.pat"), testVektor, false);
//            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
//                properties.getProperty("lzprMask.soubor.bat"));
            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("atalanta.lzpr"));
            chybMaska = DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska"));
            chybMaskaZal = DipUtils.getVysledekOperaceOR(chybMaska, chybMaskaZal);
            aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
            detekPoruchy.add(aktDetPoruch);          
            testVektor = DipUtils.krokLZPR(polynom, testVektor);            
        }

        DipUtils.zapisPocetDetekDoSouboru(detekPoruchy);
    }

   /**
    * Metoda simuluje deterministickou fazi testovani.
    *
    * @throws Exception
    */
    private void simulujDeterFazi() throws Exception {
        Properties properties = DipUtils.getProperties();
        String chybMaska = "";
        int detPoruch = Integer.valueOf(properties.getProperty("detekovatelnych.poruch"));
        String chybMaskaZal = DipUtils.setDefaultChybMaska(detPoruch);
        int aktDetPoruch = 0;
        List<Integer> detekPoruchy = new ArrayList<Integer>();

        FileReader reader = new FileReader(properties.getProperty("soubor.atalanta.pat"));
        BufferedReader bufferedReader = new BufferedReader(reader);
        String testVektor;

        DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("soubor.kompakt.bat"));

        while ((testVektor = bufferedReader.readLine()) != null) {
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.mask.pat"), testVektor, false);
            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("lzprMask.soubor.bat"));
            chybMaska = DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska"));
            chybMaskaZal = DipUtils.getVysledekOperaceOR(chybMaska, chybMaskaZal);            
            aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
            detekPoruchy.add(aktDetPoruch);
            System.out.println(aktDetPoruch);
        }
        System.out.println("KONEC!");
        reader.close();
        DipUtils.zapisPocetDetekDoSouboru(detekPoruchy);
        
    }

    /**
     * Metoda vrati sekvenci vzniklou kroky LZPR.
     *
     * @param polynom
     * @param seed
     * @return sekvence vznikla kroky LZPR
     * @throws Exception
     */
    public List<String> vytvorSekvenci(String polynom, String seed) throws Exception {
        if (!DipValidators.jePolynASeedValidni(polynom, seed)){            
            throw new Exception(DipUtils.getProperties().getProperty("seed.polynom.validace.error"));
        }

        List<String> sekvence = new ArrayList<String>();
        String aktVektor = seed;
        sekvence.add(aktVektor);
        int pocetVektoru = Integer.valueOf(DipUtils.getProperties().getProperty("maximalni.pocet.vektoru"));
        if (pocetVektoru > Math.pow(2, polynom.length()) - 1) {
            pocetVektoru = (int) (Math.pow(2, polynom.length()) - 1);
        }      
        for (int i = 0; i < pocetVektoru; i++) {
            aktVektor = DipUtils.rotujString(aktVektor);
            if (aktVektor.charAt(0) == '1') {
                aktVektor = DipUtils.xor(polynom, aktVektor);
            }
            /**
             * Opakuje se vychozi stav(seed). Generovala by se znova ta sama sekvence,
             * takze nema smysl dale pokracovat.
             */
            if (aktVektor.equals(seed)) {
                break;
            }
            sekvence.add(aktVektor);
        }        
        return sekvence;
    }    

    /**
     * Metoda vrati sekvenci vzniklou kroky LZPR.
     *
     * @param polynom
     * @param seed
     * @param pocetVektoru  
     * @return sekvence vznikla kroky LZPR
     * @throws Exception
     */
    public List<String> vytvorSekvenci(String polynom, String seed, int pocetVektoru) throws Exception {
        DipUtils.getProperties().setProperty("maximalni.pocet.vektoru", String.valueOf(pocetVektoru));
        return vytvorSekvenci(polynom, seed);
    } 
      
}